@@ -882,7 +882,11 @@ def get_system_conf_info(json=False):
     return system_conf_info
 
 
-async def get_updatable(modules=[], strategy="consensus"):
+async def get_updatable_async(modules=[], strategy="consensus"):
+    update_vers, resolution_applied, resolution_failed = get_updatable(modules=modules, strategy=strategy)
+    return update_vers, resolution_applied, resolution_failed
+
+def get_updatable(modules=[], strategy="consensus"):
     if strategy not in ("consensus", "force", "skip"):
         raise ValueError('Unknown strategy "{}"'.format(strategy))
     if not modules:
