@@ -8,6 +8,7 @@ from cravat import admin_util as au
 from cravat.config_loader import ConfigLoader
 from cravat import util
 from cravat.inout import ColumnDefinition
+from cravat.util import write_log_msg
 import subprocess
 import re
 import logging
@@ -62,6 +63,8 @@ class CravatReport:
     def parse_cmd_args(self, inargs, inkwargs):
         parsed_args = cravat.util.get_args(parser, inargs, inkwargs)
         self.parsed_args = parsed_args
+        if parsed_args.md is not None:
+            constants.custom_modules_dir = parsed_args.md
         self.dbpath = parsed_args.dbpath
         self.filterpath = parsed_args.filterpath
         self.filtername = parsed_args.filtername
@@ -99,16 +102,17 @@ class CravatReport:
         if self.filter is None:
             if self.confs is not None and "filter" in self.confs:
                 self.filter = self.confs["filter"]
+            local = au.mic.get_local()
             if (
                 self.filter is None
                 and self.filterpath is None
                 and self.filtername is None
                 and self.filterstring is None
                 and parsed_args.package is not None
-                and parsed_args.package in au.mic.local
-                and "filter" in au.mic.local[parsed_args.package].conf
+                and parsed_args.package in local
+                and "filter" in local[parsed_args.package].conf
             ):
-                self.filter = au.mic.local[parsed_args.package].conf["filter"]
+                self.filter = local[parsed_args.package].conf["filter"]
         self.output_basename = os.path.basename(self.dbpath)[:-7]
         status_fname = "{}.status.json".format(self.output_basename)
         self.status_fpath = os.path.join(self.output_dir, status_fname)
@@ -150,8 +154,21 @@ class CravatReport:
             return False
 
     async def prep(self):
-        await self.connect_db()
-        await self.load_filter()
+        try:
+            await self.connect_db()
+            await self.load_filter()
+        except Exception as e:
+            if hasattr(self, "cf"):
+                await self.cf.close_db()
+            if not hasattr(e, "notraceback") or e.notraceback != True:
+                import traceback
+                traceback.print_exc()
+                self.logger.error(e)
+            else:
+                if hasattr(self, "logger"):
+                    write_log_msg(self.logger, e)
+            e.handled = True
+            raise
 
     def _setup_logger(self):
         if hasattr(self, "no_log") and self.no_log:
@@ -762,7 +779,7 @@ class CravatReport:
                 ]:
                     continue
                 if module_name not in local_modules:
-                    if self.args.silent == False:
+                    if self.args.silent == False and module_name != 'original_input':
                         print(
                             "            [{}] module does not exist in the system. Gene level summary for this module is skipped.".format(
                                 module_name
@@ -941,6 +958,7 @@ class CravatReport:
     async def close_db(self):
         if hasattr(self, "conn") and self.conn is not None:
             await self.conn.close()
+            self.conn = None
         if self.cf is not None:
             await self.cf.close_db()
             self.cf = None
@@ -1014,9 +1032,12 @@ def run_reporter(*inargs, **inkwargs):
             )
         return
     report_types = args.reporttypes
+    if args.md is not None:
+        constants.custom_modules_dir = args.md
+    local = au.mic.get_local()
     if len(report_types) == 0:
-        if args.package is not None and args.package in au.mic.local:
-            package_conf = au.mic.local[args.package].conf
+        if args.package is not None and args.package in local:
+            package_conf = local[args.package].conf
             if "run" in package_conf and "reports" in package_conf["run"]:
                 report_types = package_conf["run"]["reports"]
     if hasattr(args, "output_dir") and args.output_dir is not None:
@@ -1056,9 +1077,13 @@ def run_reporter(*inargs, **inkwargs):
     loop = asyncio.get_event_loop()
     response = {}
     for report_type in report_types:
+        module_info = au.get_local_module_info(report_type + "reporter")
+        if module_info is None:
+            if args.silent == False:
+                print(f"Report module for {report_type} does not exist. Skipping...")
+            continue
         if args.silent == False:
             print(f"Generating {report_type} report... ", end="", flush=True)
-        module_info = au.get_local_module_info(report_type + "reporter")
         module_name = module_info.name
         spec = importlib.util.spec_from_file_location(
             module_name, module_info.script_path
@@ -1070,9 +1095,9 @@ def run_reporter(*inargs, **inkwargs):
         if module_name in module_options:
             args.conf = module_options[module_name]
         reporter = module.Reporter(args)
-        loop.run_until_complete(reporter.prep())
         response_t = None
         try:
+            loop.run_until_complete(reporter.prep())
             response_t = loop.run_until_complete(reporter.run())
             output_fns = None
             if args.silent == False:
@@ -1083,12 +1108,18 @@ def run_reporter(*inargs, **inkwargs):
                 if output_fns is not None:
                     print(f"report created: {output_fns}")
         except Exception as e:
+            if hasattr(reporter, "cf"):
+                loop.run_until_complete(reporter.cf.close_db())
+            if hasattr(e, "handled") and e.handled == True:
+                if not hasattr(e, "notraceback") or e.notraceback != True:
+                    import traceback
+                    traceback.print_exc()
+                else:
+                    if hasattr(reporter, "logger"):
+                        write_log_msg(self.logger, e)
             if args.silent == False:
-                print(f"report generation failed.")
+                print("report generation failed for {} report.".format(report_type))
             response_t = None
-            import traceback
-
-            traceback.print_exc()
         response[report_type] = response_t
     if len(report_types) == 1 and len(response) == 1:
         return response[list(response.keys())[0]]
@@ -1109,7 +1140,6 @@ parser.add_argument(
     "-t",
     dest="reporttypes",
     nargs="+",
-    choices=au.report_formats(),
     default=[],
     help="report types",
 )
@@ -1203,4 +1233,9 @@ parser.add_argument(
     help='Sample IDs to exclude',
 )
 parser.add_argument("--package", help="Use filters and report types in a package")
+parser.add_argument(
+    "--md", 
+    default=None, 
+    help="Specify the root directory of OpenCRAVAT modules (annotators, etc)"
+)
 parser.set_defaults(func=run_reporter)
